#!/usr/bin/perl
##############################################################################
# Copyright (c) 2003
# Author Sylvain Daubert <sly_@ifrance.com>
#
# This program is free software; you can redistribute it and/or modify it 
# under the terms of the GNU General Public License as published by the Free 
# Software Foundation; either version 2 of the License, or (at your option) 
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT 
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 675 Mass 
# Ave, Cambridge, MA 02139, USA.
# Sylvain Daubert <sly_@ifrance.com>
#############################################################################
## This file contains handlers for gwine UI
#############################################################################
require 5.8.0;
use warnings;

#==============================================================================
#=== This is the 'AppGwine' class                              
#==============================================================================
package AppGwine;
use strict;

BEGIN {
    use Gwine::Config;
    use Gwine::gwineUI;
}


##################################################################
## Global variables
##################################################################
our ($config, $db, $app, %pixmaps, $lh);
my ($wine_iter, $prod_iter);
my ($wine_path, $prod_path);
our (@wine_colors);


##################################################################
## File menu
##################################################################
sub on_new_event {
    my $dialog = Gtk2::MessageDialog->new($app->FORM->{app_gwine},
					  'destroy-with-parent',
					  'question',
					  'none',
					  mt("Save [_1] before creating new cellar?\n All unsaved data will be lost.", 
					     $config->opened_file));
    $dialog->add_buttons('gtk-yes', -8, 'gtk-no', -9, 'gtk-cancel', -6);
    my $return = $dialog->run;
    $dialog->destroy;
    return if $return eq 'cancel';
    # Save db if 'yes'
    on_save_event() if $return eq 'yes';
    
    # Delete all db in memory
    $db->clear;
    my $name = $config->opened_file(mt('New Cellar'));
    $app->FORM->{'app_gwine'}->set_title("Gwine - $name");
    $app->FORM->{'ztreeview'}->signal_emit('map')
}

sub open_file {
    my $filename = shift;
    $app->FORM->{appbar}->set_status(mt('Opening [_1]...', $filename));
    $db->clear;
    $db->open_db($filename);
    # Convert all dates to locale
    $app->FORM->{appbar}->set_status(mt('Converting database informations to locale...', $filename));
    my @wines = @{$db->wine};
    foreach my $wine (@wines) {
	next if $wine eq '-1';
	$wine->date($lh->save2date($wine->date)) if $wine->date;
	$wine->size($lh->numf($wine->size));
	$wine->price($lh->numf($wine->price));
	my @entries = $wine->book_get_entries;
	$_->{date} = $lh->date2save($_->{date}) foreach @entries;
    }
    $config->opened_file($filename);
    $app->FORM->{'app_gwine'}->set_title("Gwine - $filename");
    $app->FORM->{'ztreeview'}->signal_emit('map');
    $app->FORM->{appbar}->set_status('');
}

sub on_open_event {
    my $filesel = Gtk2::FileSelection->new(mt("Open file..."));
    $filesel->hide_fileop_buttons;
    my $resp = $filesel->run;
    my $filename = $filesel->get_filename if $resp eq 'ok';
    $filesel->destroy;
    return if !defined $filename;
    open_file($filename)
}

# Called by on_save_event and on_save_as_event
sub unlocalize {
    $app->FORM->{appbar}->set_status(mt('Converting locale informations for saving...'));
    my @wines = @{$db->wine};
    foreach my $wine (@wines) {
	next if $wine eq '-1';
	$wine->date($lh->date2save($wine->date)) if $wine->date;
	$wine->size($lh->numunf($wine->size), 2, 0);
	$wine->price($lh->numunf($wine->price));
	my @entries = $wine->book_get_entries;
	$_->{date} = $lh->save2date($_->{date}) foreach @entries;
    }
}

sub on_save_event {
    if (defined $config->opened_file and 
	$config->opened_file ne mt('New Cellar')) {
	unlocalize();
	$app->FORM->{appbar}->set_status(mt('Saving [_1]...', $config->opened_file));
	$db->save_db($config->opened_file)
    }
    else { on_save_as_event(@_) }
}

sub on_save_as_event {
    my $filesel = Gtk2::FileSelection->new(mt("Save file as..."));
    $filesel->set_filename($config->opened_file) if defined $config->opened_file;
    $filesel->hide_fileop_buttons;
    my $resp = $filesel->run;
    if ($resp eq 'ok') {
	my $filename = $filesel->get_filename;
	unlocalize();
	$app->FORM->{appbar}->set_status(mt('Saving [_1]...', $filename));
	$db->save_db($filename);
	$config->opened_file($filename);
	$app->FORM->{'app_gwine'}->set_title("Gwine - $filename")
    }
    $filesel->destroy;
}

sub on_properties_event {
    my $date = mt("unknown");
    if (open DATA, $config->opened_file) {
	while (<DATA>) {
	    next if ! /^date=(.*)/;
	    $date = $1;
	    last;
	}
	close DATA;
    }

    my $wine_quantity = $db->wine->true_size;
    my $bottle_quantity = 0;
    my @wines = $db->wine->list;
    foreach my $wine (@wines) {
	next if !defined $wine;
	next if $wine eq '-1';
	$bottle_quantity += $wine->number;
    }

    my $dialog = Dialog_properties->new($config->opened_file,
					$date,
					$wine_quantity,
					$bottle_quantity);
    $dialog->TOPLEVEL->show_all;
    $dialog->TOPLEVEL->run;
    $dialog->TOPLEVEL->destroy;
}

sub on_quit_event {
    if ($config->conf_popups) {
	# Popus Dialog "Yes/No"
	my $dialog = Gtk2::MessageDialog->new($app->FORM->{app_gwine},
					      'destroy-with-parent',
					      'warning',
					      'none',
					      mt('All unsaved data will be lost. Do you want to quit?'));
	$dialog->add_buttons('gtk-yes', -8, 'gtk-no', -9);
	my $return = $dialog->run;
	$dialog->destroy;
	return if $return eq 'no';
    }

    # Save config
    my ($width, $heigth) = $app->FORM->{app_gwine}->get_size;
    $config->geometry("${width}x$heigth");
    $config->paned_pos($app->FORM->{hpaned}->get_position);
    # This may be do only in on_preferences_event sub
    my @wine_visible = $app->wine_list_columns_get_visible;
    $config->wine_visible(\@wine_visible);
    my @prod_visible = $app->prod_list_columns_get_visible;
    $config->prod_visible(\@prod_visible);
    $config->save;

    Gtk2->main_quit
}


###################################################################
# Edit menu
###################################################################
sub on_preferences_event {
    my $dialog = Dialog_preferences->new;
    $dialog->FORM->{'button_lfile'}->set_active($config->save_opened_file);
    $dialog->FORM->{'button_size'}->set_active($config->save_geometry);
    $dialog->FORM->{'button_popup'}->set_active($config->conf_popups);
    my $wstore = $dialog->FORM->{'lstore1'};
    for (my $i = 0; $i < @AppGwine::wine_cols; $i++) {
	my $ins_iter = $wstore->append;
	$wstore->set($ins_iter,
		     0 => $AppGwine::wine_cols[$i],
		     1 => $config->wine_visible->[$i])
    }
    my $pstore = $dialog->FORM->{'lstore2'};
    for (my $i = 0; $i < @AppGwine::prod_cols; $i++) {
	my $ins_iter = $pstore->append;
	$pstore->set($ins_iter,
		     0 => $AppGwine::prod_cols[$i],
		     1 => $config->prod_visible->[$i])
    }
    $dialog->TOPLEVEL->show_all;
    $dialog->TOPLEVEL->run;

    $config->save_opened_file($dialog->FORM->{'button_lfile'}->get_active);
    $config->save_geometry($dialog->FORM->{'button_size'}->get_active);
    $config->conf_popups($dialog->FORM->{'button_popup'}->get_active);
    # Get wine visibility info
    my $iter = $wstore->get_iter_first;
    my $i = 0;
    my @tab = ();
    do {
	$tab[$i] = $wstore->get_value($iter, 1);
	$i++;
    } while ($iter = $wstore->iter_next($iter));
    $config->wine_visible(\@tab);
    $app->wine_list_columns_set_visible(@tab);

    # Get prod visibility info
    $iter = $pstore->get_iter_first;
    $i = 0;
    @tab = ();
    do {
	$tab[$i] = $pstore->get_value($iter, 1);
	$i++;
    } while ($iter = $pstore->iter_next($iter));
    $config->prod_visible(\@tab);
    $app->prod_list_columns_set_visible(@tab);

    $dialog->TOPLEVEL->destroy;
}


###################################################################
# Help menu
###################################################################
sub on_about_event {
    my $file = $config->icon_path . "/gwine.png";
    my $pixbuf = Gtk2::Gdk::Pixbuf->new_from_file($file);
    About_gwine->run($pixbuf);
}


###################################################################
# Gwine/Zones menu
###################################################################
sub on_new_zone_zone_event { on_new_zone_event($_[0], 0) }
sub on_new_appellation_zone_event { on_new_zone_event($_[0], 1) }
sub on_new_extension_zone_event { on_new_zone_event($_[0], 2) }

sub on_new_zone_event {
    my (undef, $action) = @_;
    my $view = $app->FORM->{'ztreeview'};
    my $store = $app->FORM->{'ztstore'};
    my (undef, $iparent) = $view->get_selection->get_selected;
    my ($parent_type, $parent_id) = split m!/!, $store->get_value($iparent, 2);
    
    # Parent is root...
    $parent_id = -1 if $parent_type eq 'WINE';

    my $type = ('zone', 'appel', 'ext')[$action];

    # Create new db item
    my $id = -1;
    my $name = '';
    if ($type eq 'zone') {
	$name = mt('New Zone');
	$id = $db->zone->add(Gwine::GZone->new($name, $parent_id));
    }
    elsif ($type eq 'appel') {
	$name = mt('New Appellation');
	$id = $db->appel->add(Gwine::GAppel->new($name, $parent_id));
    }
    elsif ($type eq 'ext') {
	$name = mt('New Extension');
	my $appel = $db->appel->by_id($parent_id);
	$id = $appel->add_extension($name);
	$id = "$parent_id:$id";
    }
    else { return }

    # Insert new cell
    my $iter = $store->append($iparent);
    $store->set($iter,
		0 => $pixmaps{$type},
		1 => $name,
		2 => "$type/$id");

    # if parent not expanded, expand !
    my $par_path = $store->get_path($iparent);
    $view->expand_row($par_path, FALSE);

    # Editing for new name
    my $path = $store->get_path($iter);
    my $column = $view->get_column(1);
    $view->set_cursor($path, $column, TRUE);
#    $view->grab_focus;
}

sub on_modify_zone_event {
    my $view = $app->FORM->{'ztreeview'};
    my $store = $app->FORM->{'ztstore'};
    my (undef, $iter) = $view->get_selection->get_selected;
    my $path = $store->get_path($iter);
    my $column = $view->get_column(1);

    $view->set_cursor($path, $column, TRUE);
#    $view->grab_focus;
}

sub on_delete_zone_event {
    my $store = $app->FORM->{'ztstore'};
    my $select = $app->FORM->{'ztreeview'}->get_selection;
    my (undef, $iter) = $select->get_selected;
    my ($type, $id) = split m!/!, $store->get_value($iter, 2);
    
    # if a subnode exists, cannot delete this node
    if ($store->iter_has_child($iter)) {
	$app->FORM->{'appbar'}->set_status(mt('This node cannot be deleted: a sub-node exists'));
	return;
    }

    # if a wine exists, cannot delete this node
    if ($type ne 'zone') {
	my $iter = $app->FORM->{'wlstore'}->get_iter_first;
	print "Iter: $iter\n";
	if (defined $iter) {
	    $app->FORM->{'appbar'}->set_status(mt('This node cannot be deleted: wines exist'));
	    return;
	}
    }
    
    if ($config->conf_popups) {
	# Popus Dialog "Yes/No"
	my $dialog = Gtk2::MessageDialog->new($app->FORM->{app_gwine},
					      'destroy-with-parent',
					      'warning',
					      'none',
					      mt('This zone will be deleted permanently.\nAre you sure?'));
	$dialog->add_buttons('gtk-yes', -8, 'gtk-no', -9);
	my $return = $dialog->run;
	$dialog->destroy;
	return if $return eq 'no';
    }

    # Delete zone from list...
    $store->remove($iter);
    # ... and from db
 SWITCH: {
	if ($type eq 'zone') { $db->zone->del_by_id($id); last }
	if ($type eq 'appel') { $db->appel->del_by_id($id); last }
	if ($type eq 'ext') { 
	    my ($appid, $extid) = split /:/, $id;
	    $db->appel->by_id($appid)->delete_extension($extid);
	    last 
	}
    }
}


###################################################################
# Gwine/Wines menu
###################################################################

# Arg : resp (ok or cancel)
# Return a ref on a Gwine::GWine
sub create_wine_from_dialog {
    my $dialog = shift;

    my $resp = $dialog->TOPLEVEL->run;

    if ($resp eq 'ok') {
	my $form = $dialog->FORM;
	my $domain = $form->{'entry_domain'}->get_text;
	my $vintage = $form->{'entry_vintage'}->get_text;
	my $color = $form->{'omenu_color'}->get_history;
	my $bdate = $form->{'entry_bdate'}->get_text;
	my $nb = $form->{'entry_quantity'}->get_text;
	my $locale_size = $form->{'entry_size'}->get_text;
	my $size = $lh->numunf($locale_size);
	my $locale_price = $form->{'entry_price'}->get_text;
	my $price = $lh->numunf($locale_price);
	my $prod_name = $form->{'combo_prod'}->entry->get_text;
	my $prod = $db->prod->by_name($prod_name);
	$prod = $db->prod->get_id($prod);
	my $cons = $form->{'entry_cons'}->get_text;

	my $text_buf = $form->{'tview_note'}->get_buffer;
	my $start_iter = $text_buf->get_start_iter;
	my $end_iter = $text_buf->get_end_iter;
	my $notes = $text_buf->get_text($start_iter, $end_iter, FALSE);

	my $selection = $app->FORM->{'ztreeview'}->get_selection;
	my $store = $app->FORM->{'ztstore'};
	my (undef, $iter) = $selection->get_selected;
	my $name = $store->get_value($iter, 1);
	my ($type, $id) = split m!/!, $store->get_value($iter, 2);

	# Some checking...
	### Check date format
	my $format = mt('\d{1,2}-\d{1,2}-\d{4}');
	if ($bdate !~ m|$format|) {
	    $form->{entry_bdate}->set_text(mt('mm-dd-yyyy'));
	    $form->{entry_bdate}->select_region(0, -1);
	    return create_wine_from_dialog($dialog)
	}

	# Create new wine
	my $wine = Gwine::GWine->new;
	$wine->set(domain => $domain, 
		   vintage => $vintage, 
		   color => $color, 
		   date => $bdate, 
		   number => $nb, 
		   size => $size, 
		   price =>$price, 
		   appellation => $id, 
		   productor => $prod,
		   conservation => $cons,
		   notes => $notes);
	return $wine;
    }
    else { # $resp eq 'cancel'
	return
    }
}

sub on_new_wine_event {
    my $dialog = Dialog_wine->new(mt("New wine"));
    my $form = $dialog->FORM;
    $form->{'entry_size'}->set_text(0.75);
    my @prods = $db->list_prod;
    $form->{'combo_prod'}->set_popdown_strings(@prods);
    $dialog->TOPLEVEL->show_all;
    my $wine = create_wine_from_dialog($dialog);
    if (defined $wine) {	
	# Add it to database
	my $wine_id = $db->wine->add($wine);
	# Add it to wine list
	$app->add_wine($wine, $db);
    }
    $dialog->TOPLEVEL->destroy;
}

sub on_modify_wine_event {
    my $store = $app->FORM->{'wlstore'};
    my $select = $app->FORM->{'wtreeview'}->get_selection;
    my (undef, $iter) = $select->get_selected;
    my $wine_domain = $store->get_value($iter, 0);
    my $wine_id =  $store->get_value($iter, 8);
    my $wine = $db->wine->by_id($wine_id);

    my $dialog = Dialog_wine->new(mt("Modify [_1] ([_2])",
				     $wine_domain,
				     $wine_id));
    my $form = $dialog->FORM;
    $form->{'entry_domain'}->set_text($wine->domain);
    $form->{'entry_vintage'}->set_text($wine->millesime);
    $form->{'omenu_color'}->set_history($wine->color);
    $form->{'entry_quantity'}->set_text($wine->number);
    $form->{'entry_size'}->set_text($wine->size);
    ### Notes
    my $text_buf = $form->{'tview_note'}->get_buffer;
    my $end_iter = $text_buf->get_end_iter;
    $text_buf->insert($end_iter, $wine->notes);

    $form->{'entry_bdate'}->set_text($wine->date);
    $form->{'entry_cons'}->set_text($wine->conservation);
    my @prods = $db->list_prod;
    $form->{'combo_prod'}->set_popdown_strings(@prods);
    if ($wine->productor == -1) {
	$form->{'combo_prod'}->entry->set_text("");
    }
    else {
	$form->{'combo_prod'}->entry->set_text($db->prod->by_id($wine->productor)->name);
    }
    $form->{'entry_price'}->set_text($wine->price);
    $dialog->TOPLEVEL->show_all;
    my $modified_wine = create_wine_from_dialog($dialog);

    if (defined $modified_wine) {
	$modified_wine->size($lh->numf($modified_wine->size, 2, 0));
	$modified_wine->price($lh->numf($modified_wine->price));
	# Add it to database
	$db->wine->by_id($wine_id, $modified_wine);
	# Suppress old wine from list
	$app->FORM->{'wlstore'}->remove($iter);
	# Add it to wine list
	$app->add_wine($modified_wine, $db);
    }
    $dialog->TOPLEVEL->destroy;
}

sub on_delete_wine_event {
    my $store = $app->FORM->{'wlstore'};
    my $select = $app->FORM->{'wtreeview'}->get_selection;
    my (undef, $iter) = $select->get_selected;
    my $wine_id =  $store->get_value($iter, 8);
    
    if ($config->conf_popups) {
	# Popus Dialog "Yes/No"
	my $dialog = Gtk2::MessageDialog->new($app->FORM->{app_gwine},
					      'destroy-with-parent',
					      'warning',
					      'none',
					      mt('This wine will be deleted permanently.\nAre you sure?'));
	$dialog->add_buttons('gtk-yes', -8, 'gtk-no', -9);
	my $return = $dialog->run;
	$dialog->destroy;
	return if $return eq 'no';
    }

    # Delete wine from list...
    $app->FORM->{'wlstore'}->remove($iter);
    # ... and from db
    $db->wine->del_by_id($wine_id);    
}


###################################################################
# Gwine/Productors menu
###################################################################

# Arg : resp (ok or cancel)
# Return a ref on a Gwine::GProd
sub create_prod_from_dialog {
    my $dialog = shift;

    my $resp = $dialog->TOPLEVEL->run;

    if ($resp eq 'ok') {
	my $form = $dialog->FORM;
	my $name = $form->{'entry_name'}->get_text;
	my $address = $form->{'entry_addr'}->get_text;
	my $zip = $form->{'entry_zip'}->get_text;
	my $city = $form->{'entry_city'}->get_text;
	my $country = $form->{'entry_country'}->get_text;

	# Create new productor
	return Gwine::GProd->new($name, $address, $zip, 
				 $city, $country);
    }
    else { # $resp eq 'cancel'
	return
    }
}

sub on_new_prod_event {
    my $dialog = Dialog_prod->new(mt("New Productor"));
    $dialog->TOPLEVEL->show_all;
    my $prod = create_prod_from_dialog($dialog);
    if (defined $prod) {	
	# Add it to database
	my $prod_id = $db->prod->add($prod);
	# Add it to prod list
	$app->add_prod($prod, $db);
    }
    $dialog->TOPLEVEL->destroy;
}

sub on_modify_prod_event {
    my $store = $app->FORM->{'plstore'};
    my $select = $app->FORM->{'ptreeview'}->get_selection;
    my $iter = $select->get_selected;
    my $prod_name = $store->get_value($iter, 0);
    my $prod_id =  $store->get_value($iter, 5);
    my $prod = $db->prod->by_id($prod_id);

    my $dialog = Dialog_prod->new(mt("Modify [_1] ([_2])", 
				     $prod_name,
				     $prod_id));
    my $form = $dialog->FORM;
    $form->{'entry_name'}->set_text($prod->name);
    $form->{'entry_addr'}->set_text($prod->address);
    $form->{'entry_zip'}->set_text($prod->zip);
    $form->{'entry_city'}->set_text($prod->city);
    $form->{'entry_country'}->set_text($prod->country);
    $dialog->TOPLEVEL->show_all;
    my $modified_prod = create_prod_from_dialog($dialog);
    if (defined $modified_prod) {	
	# Add it to database
	$db->prod->by_id($prod_id, $modified_prod);
	# Suppress old prod from list
	$app->FORM->{'plstore'}->remove($iter);
	# Add it to prod list
	$app->add_prod($modified_prod, $db);
    }
    $dialog->TOPLEVEL->destroy;
}

sub on_delete_prod_event {
    my $store = $app->FORM->{'plstore'};
    my $select = $app->FORM->{'ptreeview'}->get_selection;
    my (undef, $iter) = $select->get_selected;
    my $prod_id =  $store->get_value($iter, 5);
    
    if ($config->conf_popups) {
	# Popus Dialog "Yes/No"
	my $dialog = Gtk2::MessageDialog->new($app->FORM->{app_gwine},
					      'destroy-with-parent',
					      'warning',
					      'none',
					      mt('This productor will be deleted permanently.\nAre you sure?'));
	$dialog->add_buttons('gtk-yes', -8, 'gtk-no', -9);
	my $return = $dialog->run;
	$dialog->destroy;
	return if $return eq 'no';
    }

    # Delete prod from list...
    $app->FORM->{'plstore'}->remove($iter);
    # ... and from db
    $db->prod->del_by_id($prod_id);    
}


######################################################################
## Zone Tree Signals
######################################################################

sub on_ztreeview_map {
    my $tview = shift;
    my $store = $app->FORM->{'ztstore'};
    
    $store->clear;
    # Sort function ?

    $wine_iter = $store->append(undef);
    $wine_path = $store->get_path($wine_iter)->to_string;
    $store->set($wine_iter,
		0 => $pixmaps{vines},
		1 => mt('Wines'),
		2 => 'WINE');
    $prod_iter = $store->append(undef);
    $prod_path = $store->get_path($prod_iter)->to_string;
    $store->set($prod_iter,
		0 => $pixmaps{prod},
		1 => mt('Productors'),
		2 => 'PROD');

    my @lzone = $db->zone->list;
    my @zone_nodes = ();
    while (@lzone) {
	my @list = @lzone;
	@lzone = ();
	foreach my $zone (@list) {
	    if ($zone eq '-1') { next }
	    if ($zone->parent eq '-1') {
		my $id = $db->zone->get_id($zone);
		$zone_nodes[$id] = $store->append($wine_iter);
		$store->set($zone_nodes[$id],
			    0 => $pixmaps{zone},
			    1 => $zone->zone,
			    2 => "zone/$id");
	    }    
	    else {
		my $parent = $zone_nodes[$zone->parent];
		# Here, if parent is not still created, creation of child is delayed
		if (! defined $parent) { push @lzone, $zone; next }
		my $id = $db->zone->get_id($zone);
		$zone_nodes[$id] = $store->append($parent);
		$store->set($zone_nodes[$id], 
			    0 => $pixmaps{zone},
			    1 => $zone->zone,
			    2 => "zone/$id");
	    }
	}
    }
    my @lappel = $db->appel->list;
    foreach my $appel (@lappel) {
	if ($appel eq -1) { next }
	my $name = $appel->appellation;
	my $id = $db->appel->get_id($appel);
	my $appel_node = $store->append($zone_nodes[$appel->vineyard]);
	$store->set($appel_node, 
		    0 => $pixmaps{appel},
		    1 => $name,
		    2 => "appel/$id:");
	# For this appellation, we create its extensions
	if (defined $appel->extensions) {
	    foreach my $extension (sort @{$appel->extensions}) {
		if ($extension ne 'null') {
		    my $ext_id = $appel->get_extension_id( $extension );
		    $store->set($store->append($appel_node),
				0 => $pixmaps{ext},
				1 => $extension,
				2 => "ext/$id:$ext_id");
		}
	    }
	}
    }
    # Expand first Wines node
    $tview->expand_row($store->get_path($wine_iter), FALSE);
#    $ctree->sort;
#    $ctree->set_auto_sort($true);
}

sub on_ztreeview_button_press_event {
    my ($class, $event, $data) = @_;

    my $store = $class->get_model;
    my $select = $class->get_selection;
    my (undef, $iter) = $select->get_selected;

    if ($event->button == 3) { # Left mouse click (or middle...)
	my $itemfac = $app->FORM->{'itemfac1'};
	my $path = $store->get_path($iter)->to_string;
	my $menu;
	if ($path eq $prod_path) {
	    $menu = $app->FORM->{gwine_menu}->{subtree}->[1]->{widget}->get_submenu;
	}
	else {
	    $menu = $app->FORM->{gwine_menu}->{subtree}->[2]->{widget}->get_submenu;
	}
	$menu->popup(undef, undef, undef, undef, 
		     $event->button, $event->time);
	return 1
    }
    else { # Selection
	$config->last_clicked('zonetree');
    }
    return 0;
}

sub on_ztreeview_selection_changed {
    my ($class, $event, $data) = @_;
    
    my (undef, $iter) = $class->get_selected;
    my $store = $class->get_tree_view->get_model;

    return 0 if !defined $iter;
    my $path = $store->get_path($iter)->to_string;
    
    my $wmenu = $app->FORM->{gwine_menu}->{subtree}->[0]->{widget};
    my $wmod = $app->FORM->{gwine_menu}->{subtree}->[0]->{subtree}->[1]->{widget};
    my $wdel = $app->FORM->{gwine_menu}->{subtree}->[0]->{subtree}->[2]->{widget};
    my $pmenu = $app->FORM->{gwine_menu}->{subtree}->[1]->{widget};
    my $pmod = $app->FORM->{gwine_menu}->{subtree}->[1]->{subtree}->[1]->{widget};
    my $pdel = $app->FORM->{gwine_menu}->{subtree}->[1]->{subtree}->[2]->{widget};
    my $zmenu = $app->FORM->{gwine_menu}->{subtree}->[2]->{widget};
    my $znew = $app->FORM->{gwine_menu}->{subtree}->[2]->{subtree}->[0]->{widget};
    my $znew_zone = $app->FORM->{gwine_menu}->{subtree}->[2]->{subtree}->[0]->{subtree}->[0]->{widget};
    my $znew_app = $app->FORM->{gwine_menu}->{subtree}->[2]->{subtree}->[0]->{subtree}->[1]->{widget};
    my $znew_ext = $app->FORM->{gwine_menu}->{subtree}->[2]->{subtree}->[0]->{subtree}->[2]->{widget};
    my $zmod = $app->FORM->{gwine_menu}->{subtree}->[2]->{subtree}->[1]->{widget};
    my $zdel = $app->FORM->{gwine_menu}->{subtree}->[2]->{subtree}->[2]->{widget};

    # When selection changed in tree, in new wine or prod list, nothing
    # is selected. so we cannot delete or modify an item.
    $wmod->set_sensitive(FALSE);
    $wdel->set_sensitive(FALSE);
    $pmod->set_sensitive(FALSE);
    $pdel->set_sensitive(FALSE);

    # Productor's iter ?
    if ($path eq $prod_path) {
	$app->FORM->{'list_label'}->set_text(mt('Productors'));
	$app->FORM->{'notebook1'}->set_current_page(1);
	list_prods();
	# Set menus
	$wmenu->set_sensitive(FALSE);
	$pmenu->set_sensitive(TRUE);
	$zmenu->set_sensitive(FALSE);	
	return 0;
    }
    
    if ($path eq $wine_path) {
	$app->FORM->{'list_label'}->set_text(mt('Wines'));
	$app->FORM->{'notebook1'}->set_current_page(2);
	# Set menus
	$wmenu->set_sensitive(FALSE);
	$pmenu->set_sensitive(FALSE);
	$zmenu->set_sensitive(TRUE);	
	$znew_zone->set_sensitive(TRUE);	
	$znew_app->set_sensitive(FALSE);	
	$znew_ext->set_sensitive(FALSE);	
	$zmod->set_sensitive(FALSE);	
	$zdel->set_sensitive(FALSE);	
    }
    else {
	my $name = $store->get_value($iter, 1);
	my ($type, $id) = split m!/!, $store->get_value($iter, 2);
	my $label = $app->FORM->{'list_label'};
	if ($type eq 'ext') {
	    my ($appel_id) = split /:/, $id;
	    my $appel = $db->appel->by_id($id);
	    my $appellation = $appel->appellation;
	    my $zone = $db->get_zone($appel->vineyard)->zone;
	    $label->set_text("$zone/$appellation $name");
	    list_wines();
	    $app->FORM->{'notebook1'}->set_current_page(0);
	    # Set menus
	    $wmenu->set_sensitive(TRUE);
	    $pmenu->set_sensitive(FALSE);
	    $zmenu->set_sensitive(TRUE);	
	    $znew->set_sensitive(FALSE);
	    $zmod->set_sensitive(TRUE);	
	    $zdel->set_sensitive(TRUE);	
	}
	elsif ($type eq 'appel') {
	    my $appel = $db->appel->by_id($id);
	    my $zone = $db->get_zone($appel->vineyard);
	    my $zones = $zone->zone;
	    while ($zone->parent != -1) {
		$zone = $db->get_zone($zone->parent);
		$zones = $zone->zone . '/' . $zones;
	    }
	    $label->set_text("$zones/$name");
	    list_wines();
	    $app->FORM->{'notebook1'}->set_current_page(0);
	    # Set menus
	    $wmenu->set_sensitive(TRUE);
	    $pmenu->set_sensitive(FALSE);
	    $zmenu->set_sensitive(TRUE);	
	    $znew->set_sensitive(TRUE);
	    $znew_zone->set_sensitive(FALSE);	
	    $znew_app->set_sensitive(FALSE);	
	    $znew_ext->set_sensitive(TRUE);	
	    $zmod->set_sensitive(TRUE);
	    $zdel->set_sensitive(TRUE);
	}
	elsif ($type eq 'zone') {
	    my $zone = $db->get_zone($id);
	    my $zones = $name;
	    while ($zone->parent != -1) {
		$zone = $db->get_zone($zone->parent);
		$zones = $zone->zone . '/' . $zones;
	    }
	    $label->set_text($zones);
	    $app->FORM->{'notebook1'}->set_current_page(2);
	    # Set menus
	    $wmenu->set_sensitive(FALSE);
	    $pmenu->set_sensitive(FALSE);
	    $zmenu->set_sensitive(TRUE);	
	    $znew->set_sensitive(TRUE);
	    $znew_zone->set_sensitive(TRUE);	
	    $znew_app->set_sensitive(TRUE);	
	    $znew_ext->set_sensitive(FALSE);	
	    $zmod->set_sensitive(TRUE);
	    $zdel->set_sensitive(TRUE);
	}
    }
    return 0;
}

sub on_renderer_text_edited_event {
    my $cell = shift;
    my $path = shift;
    my $new_name = shift;
    
    my $store = $app->FORM->{'ztstore'};
    my $iter = $store->get_iter_from_string($path);
    return 0 if !defined $iter;

    my ($type,$id) = split m!/!, $store->get_value($iter, 2);

    if ($type eq 'zone') {
	# Zone
	my $zone = $db->zone->by_id($id);
	$zone->zone($new_name);
    }
    elsif ($type eq 'appel') {
	# Appellation
	my ($appel_id) = split /:/, $id;
	my $appel = $db->appel->by_id($appel_id);
	$appel->appellation($new_name);
    }
    elsif ($type eq 'ext') {
	# Extension
	my ($appel_id, $ext_id) = split /:/, $id;
	my $appel = $db->appel->by_id($appel_id);
	$appel->set_extension_id($ext_id, $new_name);
    }
    else { return }

    $store->set($iter, 1, $new_name);
}


######################################################################
## Wine List Signals
######################################################################
sub on_wtreeview_button_press_event {
    my ($class, $event, $data) = @_;

    my $store = $class->get_model;
    my $select = $class->get_selection;
    my (undef, $iter) = $select->get_selected;

    if ($event->button == 3) { # Left mouse click (or middle...)
	my $itemfac = $app->FORM->{'itemfac1'};
	my $menu = $app->FORM->{gwine_menu}->{subtree}->[0]->{widget};
	$menu = $menu->get_submenu;
	$menu->popup(undef, undef, undef, undef, 
		     $event->button, $event->time);
	return 1
    }
    else { # Selection
	$config->last_clicked('winelist');
    }
    return 0;
}


sub on_wtreeview_selection_changed {
    my ($class, $event, $data) = @_;
    
    my (undef, $iter) = $class->get_selected;
    my $store = $class->get_tree_view->get_model;
    my $itemfac = $app->FORM->{'itemfac1'};
    my $mgwine = $app->FORM->{gwine_menu};
    if (!defined $iter) {
	# If $iter is undefined, there's nothing selected
	# so, we cannot modify or delete anything
	$mgwine->{subtree}->[0]->{subtree}->[1]->{widget}->set_sensitive(FALSE);
	$mgwine->{subtree}->[0]->{subtree}->[2]->{widget}->set_sensitive(FALSE);
	return 0
    }
    else {
	$mgwine->{subtree}->[0]->{subtree}->[1]->{widget}->set_sensitive(TRUE);
	$mgwine->{subtree}->[0]->{subtree}->[2]->{widget}->set_sensitive(TRUE);
    }
    my $path = $store->get_path($iter)->to_string;
    
}


######################################################################
## Wine List Subs
######################################################################
sub list_wines {
    my $list_store = $app->FORM->{'wlstore'};
    my $tree_store = $app->FORM->{'ztstore'};

    my (undef, $iter) = $app->FORM->{'ztreeview'}->get_selection->get_selected;

    # Go back if $iter is not defined
    return 0 if !defined $iter;
    my $path = $tree_store->get_path($iter)->to_string;
    # Flush list
    $list_store->clear;

    # Go back if Productor node is selected
    return 0 if $path eq $prod_path;
    # Go back if Wine node is selected
    return 0 if $path eq $wine_path;

    # Appellation node selected (include extension nodes)?
    my ($appel_ref, $appel_num, $appel_str);

    my ($type, $id) = split m!/!, $tree_store->get_value($iter, 2);
    my $label = $app->FORM->{'list_label'};
    
    my @lwine = $db->wine->list;
    foreach my $wine (@lwine) {
	next if ($wine eq -1);
	if ($wine->appellation eq $id) {
	    $app->add_wine($wine, $db)
	}
    }
    return 0;
}


######################################################################
## Productor List Signals
######################################################################
sub on_ptreeview_button_press_event {
    my ($class, $event, $data) = @_;

    my $store = $class->get_model;
    my $select = $class->get_selection;
    my (undef, $iter) = $select->get_selected;

    if ($event->button == 3) { # Left mouse click (or middle...)
	my $itemfac = $app->FORM->{'itemfac1'};
	my $menu = $app->FORM->{gwine_menu}->{subtree}->[1]->{widget}->get_submenu;
	$menu->popup(undef, undef, undef, undef, 
		     $event->button, $event->time);
	return 1
    }
    else { # Selection
	$config->last_clicked('prodlist');
    }
    return 0;
}


sub on_ptreeview_selection_changed {
    my ($class, $event, $data) = @_;
    
    my (undef, $iter) = $class->get_selected;
    my $store = $class->get_tree_view->get_model;
    my $itemfac = $app->FORM->{'itemfac1'};
    my $mgwine = $app->FORM->{gwine_menu};

    if (!defined $iter) {
	# If $iter is undefined, there's nothing selected
	# so, we cannot modify or delete anything
	$mgwine->{subtree}->[1]->{subtree}->[1]->{widget}->set_sensitive(FALSE);
	$mgwine->{subtree}->[1]->{subtree}->[2]->{widget}->set_sensitive(FALSE);
	return 0
    }
    else {
	$mgwine->{subtree}->[1]->{subtree}->[1]->{widget}->set_sensitive(TRUE);
	$mgwine->{subtree}->[1]->{subtree}->[2]->{widget}->set_sensitive(TRUE);
    }
}


######################################################################
## Prod List Subs
######################################################################
sub list_prods {
    my $list_store = $app->FORM->{'plstore'};

    # Flush list
    $list_store->clear;

    # Set prod items in list
    my @lprod = $db->prod->list;
    foreach my $prod (@lprod) {
	next if ($prod eq -1);
	$app->add_prod($prod, $db);
    }
    return 0;
}




#==============================================================================
#=== This is the 'Dialog_preferences' class                              
#==============================================================================
package Dialog_preferences;
use strict;

BEGIN {
    use Gwine::Config;
    use Gwine::gwineUI;
}


sub on_toggled_event {
  my ($cell, $path_str, $model) = @_;
  my $path = Gtk2::TreePath->new ($path_str);
    
  # get toggled iter
  my $iter = $model->get_iter ($path);
  my ($value) = $model->get ($iter, 1);
  # do something with the value
  $value ^= 1;
  # set new value
  $model->set ($iter, 1, $value);
}



1;

__END__
