#!/usr/bin/perl
##############################################################################
# Copyright (c) 2003
# Author Sylvain Daubert <sly_@ifrance.com>
#
# This program is free software; you can redistribute it and/or modify it 
# under the terms of the GNU General Public License as published by the Free 
# Software Foundation; either version 2 of the License, or (at your option) 
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT 
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 675 Mass 
# Ave, Cambridge, MA 02139, USA.
# Sylvain Daubert <sly_@ifrance.com>
#############################################################################
## This file classes for UI generation
#############################################################################
require 5.8.0;
use strict;
use warnings;

use constant FALSE => 0;
use constant TRUE  => 1;

#######################################################################
# Generic class
# Base for UI classes
#######################################################################
package Generic;

BEGIN {
    use vars qw( @ISA %fields $VERSION $AUTOLOAD $permitted_fields );
    $permitted_fields = '_permitted_fields';
}

use constant PACKAGE => 'Gwine';
use constant VERSION => '0.7.1';

$VERSION = '0.7.1'; # MakeMaker must find here a $VERSION variable.

%fields = (
	   TOPLEVEL  => undef,
	   FORM      => undef,
	   PACKAGE   => PACKAGE,
	   VERSION   => $VERSION,
	   AUTHORS   => ['Sylvain Daubert <sly_@ifrance.com>'],
	   COPYRIGHT => '(c) 2003 Sylvain Daubert'
	  );

our $lh;

# AUTOLOAD for fields in %fields
sub AUTOLOAD {
    my $self = shift;
    my $type = ref($self)
        or die "$self is not an object so we cannot '$AUTOLOAD'\n",
            "We were called from ".join(", ", caller)."\n\n";
    my $name = $AUTOLOAD;
    $name =~ s/.*://;

    if (exists $self->{$permitted_fields}{$name} ) {
        if (@_) {
            return $self->{$name} = shift;
        } else {
            return $self->{$name};
        }

    }  else {
        die "Can't access method `$name' in class $type\n".
            "We were called from ".join(", ", caller)."\n\n";
    }
}


sub new {
    my $that = shift;
    my $class = ref($that) || $that;
    my $self = { $permitted_fields   => \%fields, %fields };
    bless $self, $class;
    return $self;
}


sub run {
    my $that = shift;
    my $class = ref($that) || $that;
    my $self = $class->new(@_);
    $self->TOPLEVEL->show_all;
    return $self;
}


sub DESTROY {}


#######################################################################
# app_gwine class
#######################################################################
package AppGwine;

BEGIN {
    use Gnome2;
    use base qw( Generic );
}

our (@wine_colors, @wine_cols, @prod_cols, $lh);


sub new {
    my $that = shift;
    my $class = ref($that) || $that;
    my $self = AppGwine->SUPER::new();
    bless $self, $class;

    # Construct Gnome app
    my ($form);
    @wine_colors = (mt('White'), mt('Rose'), mt('Red'));

    $form->{'app_gwine'} = Gnome2::App->new($self->PACKAGE, 
					    $self->PACKAGE);
    $form->{'app_gwine'}->signal_connect('delete_event',
					"$class\::on_quit_event");
    $form->{'app_gwine'}->set_size_request(300,200);
    $self->TOPLEVEL($form->{'app_gwine'});

    # Get an accelgroup
    $form->{'accel_group'} = Gtk2::AccelGroup->new;

    # Create an AppBar
    $form->{'appbar'} = Gnome2::AppBar->new(TRUE, TRUE, 'never');
    $form->{'app_gwine'}->set_statusbar($form->{'appbar'});

    # Construct Main Menu

    my $mfile =  {
		  type    => 'subtree',
		  label   => '_File',
		  subtree => [  # officially 'moreinfo'
			      {
			       type            => 'item',
			       label           => '_New',
			       pixmap_type     => 'stock',
			       pixmap_info     => 'gtk-new',
			       callback        => "$class\::on_new_event",
			      },
			      {       type            => 'separator' },
			      {
			       type            => 'item',
			       label           => '_Open...',
			       pixmap_type     => 'stock',
			       pixmap_info     => 'gtk-open',
			       callback        => "$class\::on_open_event",
			      },
			      {
			       type            => 'item',
			       label           => '_Save',
			       pixmap_type     => 'stock',
			       pixmap_info     => 'gtk-save',
			       callback        => "$class\::on_save_event",
			      },
			      {
			       type            => 'item',
			       label           => 'Save _As...',
			       pixmap_type     => 'stock',
			       pixmap_info     => 'gtk-save-as',
			       callback        => "$class\::on_save_as_event",
			      },
			      {       type            => 'separator' },
			      {
			       type            => 'item',
			       label           => 'Print Previe_w...',
			       pixmap_type     => 'stock',
			       pixmap_info     => 'gtk-print-preview',
			       callback        => sub { warn 'print preview' },
			      },
			      {
			       type            => 'item',
			       label           => '_Print...',
			       pixmap_type     => 'stock',
			       pixmap_info     => 'gtk-print',
			       callback        => sub { warn 'print' },
			      },
			      {       type            => 'separator' },
			      {
			       type            => 'item',
			       label           => 'Proper_ties',
			       pixmap_type     => 'stock',
			       pixmap_info     => 'gtk-properties',
			       callback        => "$class\::on_properties_event",
			      },
			      {       type            => 'separator' },
			      {
			       type            => 'item',
			       label           => '_Quit',
			       pixmap_type     => 'stock',
			       pixmap_info     => 'gtk-quit',
			       callback        => "$class\::on_quit_event",
			      },
			     ],
		 };
    my $medit = {	  
		 type    => 'subtree',
		 label   => '_Edit',
		 subtree => [
			     {
			      type        => 'item',
			      label       => 'Prefere_nces',
			      pixmap_type => 'stock',
			      pixmap_info => 'gnome-preferences',
			      callback    => "$class\::on_preferences_event",
			     }
			    ]
		};
    my $mview = {	  
		 type    => 'subtree',
		 label   => '_View',
		 subtree => []
		};
    my $mhelp = {	  
		 type    => 'subtree',
		 label   => '_Help',
		 subtree => [
			     {
			      type        => 'item',
			      label       => '_About',
			      pixmap_type => 'stock',
			      pixmap_info => 'gnome-about',
			      callback    => "$class\::on_about_event",
			     }
			    ]
		};

    my $mwines = {
		  type    => 'subtree',
		  label   => mt('_Wines'),
		  subtree => [
			      {
			       type        => 'item',
			       label       => '_New',
			       callback    => "$class\::on_new_wine_event",
			      },
			      {
			       type        => 'item',
			       label       => mt('_Modify'),
			       callback    => "$class\::on_modify_wine_event",
			      },
			      {
			       type        => 'item',
			       label       => mt('_Delete'),
			       callback    => "$class\::on_delete_wine_event",
			      },
			     ]
		 };
    my $mprod = {
		  type    => 'subtree',
		  label   => mt('_Productors'),
		  subtree => [
			      {
			       type        => 'item',
			       label       => '_New',
			       callback    => "$class\::on_new_prod_event",
			      },
			      {
			       type        => 'item',
			       label       => mt('_Modify'),
			       callback    => "$class\::on_modify_prod_event",
			      },
			      {
			       type        => 'item',
			       label       => mt('_Delete'),
			       callback    => "$class\::on_delete_prod_event",
			      },
			     ]
		};
    my $mzones = {
		  type    => 'subtree',
		  label   => mt('_Zones'),
		  subtree => [
			      {
			       type        => 'subtree',
			       label       => '_New',
			       subtree     => [
					       {
						type      => 'item',
						label     => mt('Geographical _Zone'),
						callback  => "$class\::on_new_zone_zone_event",
					       },
					       {
						type      => 'item',
						label     => mt('_Appellation'),
						callback  => "$class\::on_new_appellation_zone_event",
					       },
					       {
						type      => 'item',
						label     => mt('_Extension'),
						callback  => "$class\::on_new_extension_zone_event",
					       },
					      ]
			      },
			      {
			       type        => 'item',
			       label       => mt('_Modify'),
			       callback    => "$class\::on_modify_zone_event",
			      },
			      {
			       type        => 'item',
			       label       => mt('_Delete'),
			       callback    => "$class\::on_delete_zone_event",
			      },
			     ]
		 };
    my $mgwine = {	  
		  type    => 'subtree',
		  label   => '_Gwine',
		  subtree => [$mwines, $mprod, $mzones]
		};

    $form->{'file_menu'} = $mfile;
    $form->{'edit_menu'} = $medit;
    $form->{'view_menu'} = $mview;
    $form->{'gwine_menu'} = $mgwine;
    $form->{'help_menu'} = $mhelp;
    $form->{'app_gwine'}->create_menus( [$mfile, $medit, $mview, 
					 $mgwine, $mhelp] );

    #####################
    ## For now, some item menus are disabled
    $mfile->{subtree}->[6]->{widget}->set_sensitive(FALSE);
    $mfile->{subtree}->[7]->{widget}->set_sensitive(FALSE);


    # Construct Toolbar
    $form->{'toolbar'} = 
	[
	 {
	  type        => 'item',
	  label       => 'Open',
	  hint        => 'Open a wine cellar',
	  pixmap_type => 'stock',
	  pixmap_info => 'gtk-open',
	  callback    => "$class\::on_open_event",
	 },
	 {
	  type        => 'item',
	  label       => 'Save',
	  hint        => 'Save wine cellar',
	  pixmap_type => 'stock',
	  pixmap_info => 'gtk-save',
	  callback    => "$class\::on_save_event",
	 },
	 {
	  type        => 'item',
	  label       => 'Quit',
	  hint        => 'Quit Gwine',
	  pixmap_type => 'stock',
	  pixmap_info => 'gtk-quit',
	  callback    => "$class\::on_quit_event",
	 },
	];
    $form->{'app_gwine'}->create_toolbar($form->{'toolbar'});

    # Construct content
    $form->{'hpaned'} = Gtk2::HPaned->new;

    # Left Part : Geographical zone Tree (in a scrolled window)
    $form->{'vbox1'} = Gtk2::VBox->new(FALSE, 0);
    $form->{'tree_label'} = Gtk2::Label->new_with_mnemonic(mt('_Appellations'));
    $form->{'tree_label'}->set_alignment(0, 0.5);
    $form->{'vbox1'}->pack_start($form->{'tree_label'}, FALSE, 
				 FALSE, 2);
    $form->{'scrolled1'} = Gtk2::ScrolledWindow->new;
    $form->{'vbox1'}->pack_start($form->{'scrolled1'}, TRUE, 
				 TRUE, 0);
    $form->{'scrolled1'}->set_policy('automatic', 'automatic');
    $form->{'ztstore'} = Gtk2::TreeStore->new('Gtk2::Gdk::Pixbuf', 'Glib::String', 
					      'Glib::String');
    $form->{'ztreeview'} = Gtk2::TreeView->new_with_model($form->{'ztstore'});
    $form->{'tree_label'}->set_mnemonic_widget($form->{'ztreeview'});
    my $renderer1 = Gtk2::CellRendererPixbuf->new;
    my $col1 = Gtk2::TreeViewColumn->new_with_attributes('', $renderer1, 'pixbuf', 0);
    $form->{'ztreeview'}->append_column($col1);
    my $renderer2 = Gtk2::CellRendererText->new;
    $renderer2->set('editable', TRUE);
    $renderer2->signal_connect('edited', "$class\::on_renderer_text_edited_event");
    my $col2 = Gtk2::TreeViewColumn->new_with_attributes('', $renderer2, 'text', 1);
    $form->{'ztreeview'}->append_column($col2);
    $form->{'ztreeview'}->get_selection->set_mode('single');
    $form->{'ztreeview'}->signal_connect('map',	"$class\::on_ztreeview_map");
    $form->{'ztreeview'}->signal_connect('button-press-event', "$class\::on_ztreeview_button_press_event");
    my $select = $form->{'ztreeview'}->get_selection;
    $select->signal_connect('changed', "$class\::on_ztreeview_selection_changed");
    $form->{'scrolled1'}->add($form->{'ztreeview'});
    $form->{'hpaned'}->add1($form->{'vbox1'});

    # Right Part : Wine and Prod lists
    # First, create a VBox to contain a label and a notebook
    $form->{'vbox2'} = Gtk2::VBox->new(FALSE, 0);
    $form->{'list_label'} = Gtk2::Label->new;
    $form->{'list_label'}->set_alignment(0, 0.5);
    $form->{'vbox2'}->pack_start($form->{'list_label'}, FALSE, 
				 FALSE, 2);
    $form->{'notebook1'} = Gtk2::Notebook->new;
    $form->{'notebook1'}->set_show_tabs(0);
    $form->{'vbox2'}->pack_start($form->{'notebook1'}, TRUE, 
				 TRUE, 0);
    # Wine List
    $form->{'scrolled2'} = Gtk2::ScrolledWindow->new;
    $form->{'scrolled2'}->set_policy('automatic', 'automatic');
    @wine_cols = (mt('Domain'), mt('Vintage'), mt('Color'), mt('Number'),
		  mt('Price'), mt('Buying date'), mt('Size'), 
		  mt('Productor'), 'Wine_id');
    my @col_types = ('Glib::String', 'Glib::String', 'Glib::String', 'Glib::Int', 
		   'Glib::String', 'Glib::String', 'Glib::String', 'Glib::String', 
		     'Glib::String');
    $form->{'wlstore'} = Gtk2::ListStore->new(@col_types);
    $form->{'wtreeview'} = Gtk2::TreeView->new_with_model($form->{'wlstore'});
    # add all columns but last to View : last column of model is only 
    # for internal purpose
    for (my $i = 0; $i < @wine_cols-1; $i++) {
	my $renderer = Gtk2::CellRendererText->new;
	my $col = Gtk2::TreeViewColumn->new_with_attributes($wine_cols[$i], 
							    $renderer,
							    'text', $i);
	$form->{'wtreeview'}->append_column($col);
    }
    $select = $form->{'wtreeview'}->get_selection;
    $select->set_mode('single');
    $select->signal_connect('changed', "$class\::on_wtreeview_selection_changed");
    $form->{'wtreeview'}->signal_connect('button-press-event', "$class\::on_wtreeview_button_press_event");
    $form->{'scrolled2'}->add($form->{'wtreeview'});
    $form->{'notebook1'}->append_page($form->{'scrolled2'}, 
				      Gtk2::Label->new(''));
    # Prod List
    $form->{'scrolled3'} = Gtk2::ScrolledWindow->new;
    $form->{'scrolled3'}->set_policy('automatic', 'automatic');
    @prod_cols = (mt('Name'), mt('Address'), mt('Zip Code'), mt('City'), 
		  mt('Country'), 'Prod_id');
    @col_types = ('Glib::String', 'Glib::String', 'Glib::String',
		  'Glib::String', 'Glib::String', 'Glib::Int');
    $form->{'plstore'} = Gtk2::ListStore->new(@col_types);
    $form->{'ptreeview'} = Gtk2::TreeView->new_with_model($form->{'plstore'});
    # add all columns but last to View : last column of model is only 
    # for internal purpose
    for (my $i = 0; $i < @prod_cols-1; $i++) {
	my $renderer = Gtk2::CellRendererText->new;
	my $col = Gtk2::TreeViewColumn->new_with_attributes($prod_cols[$i], 
							    $renderer,
							    'text', $i);
	$form->{'ptreeview'}->append_column($col);
    }
    $select = $form->{'ptreeview'}->get_selection;
    $select->set_mode('single');
    $select->signal_connect('changed', "$class\::on_ptreeview_selection_changed");
    $form->{'ptreeview'}->signal_connect('button-press-event', "$class\::on_ptreeview_button_press_event");
    $form->{'scrolled3'}->add($form->{'ptreeview'});
    $form->{'notebook1'}->append_page($form->{'scrolled3'},
				      Gtk2::Label->new(''));

    # Third page (blank) to show when prod nor app/ext selected
    $form->{'notebook1'}->append_page(Gtk2::Label->new(''),
				      Gtk2::Label->new(''));
    $form->{'notebook1'}->set_current_page(2);

    $form->{'hpaned'}->add2($form->{'vbox2'});
    $form->{'app_gwine'}->set_contents($form->{'hpaned'});

    # Add accel group
    $form->{'app_gwine'}->add_accel_group($form->{'accel_group'});


    $self->FORM($form);

    return $self;
}

# Add a Gwine::GWine in Wine List
sub add_wine {
    my $self = shift;
    my ($wine, $db) = @_;

    my $prod = $db->prod->by_id($wine->productor)->name;
    my $wine_id = $db->wine->get_id($wine);

    my $ins_iter = $self->FORM->{'wlstore'}->append;
    $self->FORM->{'wlstore'}->set($ins_iter,
				  0 => $wine->domain,
				  1 => $wine->millesime,
				  2 => $wine_colors[$wine->color],
				  3 => $wine->number,
				  4 => $lh->numf($wine->price),
				  5 => $wine->date,
				  6 => $lh->numf($wine->size, 0),
				  7 => $prod,
				  8 => $wine_id);
}


# Add a Gwine::GProd in Productor List
sub add_prod {
    my $self = shift;
    my ($prod, $db) = @_;

    my $ins_iter = $self->FORM->{'plstore'}->append;
    my $id = $db->prod->get_id($prod);
    $self->FORM->{'plstore'}->set($ins_iter,
				  0 => $prod->name,
				  1 => $prod->address,
				  2 => $prod->zip,
				  3 => $prod->city,
				  4 => $prod->country,
				  5 => $id
				 );
}


# Set visibility for wine list columns
sub wine_list_columns_set_visible {
    my ($self, @visibility) = @_;

    my @columns = $self->FORM->{wtreeview}->get_columns;
    foreach my $col (@columns) { $col->set_visible(shift @visibility) }
}


# Get visibility for wine list columns
sub wine_list_columns_get_visible {
    my $self = shift;

    my @columns = $self->FORM->{wtreeview}->get_columns;
    return map $_->get_visible, @columns
}


# Set visibility for prod list columns
sub prod_list_columns_set_visible {
    my ($self, @visibility) = @_;

    my @columns = $self->FORM->{ptreeview}->get_columns;
    foreach my $col (@columns) { $col->set_visible(shift @visibility) }
}


# Get visibility for prod list columns
sub prod_list_columns_get_visible {
    my $self = shift;

    my @columns = $self->FORM->{ptreeview}->get_columns;
    return map $_->get_visible, @columns
}


#######################################################################
# about_gwine class
#######################################################################
package About_gwine;

BEGIN {
    use Gnome2;
    use base qw( Generic );
}


sub mt(@) {$AppGwine::lh->maketext(@_)}

sub new {
    my $that = shift;
    my $class = ref($that) || $that;
    my $self = About_gwine->SUPER::new();
    bless $self, $class;

    my $pixbuf = shift;

    # Construct Gnome about
    my ($form);
    $form->{'about_gwine'} = Gnome2::About->new(
						$self->PACKAGE,
						$self->VERSION,
						$self->COPYRIGHT,
						mt('a wine cellar manager'),
						$self->AUTHORS,
						[], #documenters
						mt(${$self->AUTHORS}[0]), # Translator
						$pixbuf);
    $self->FORM($form);
    $self->TOPLEVEL($form->{'about_gwine'});
    return $self;
}


#######################################################################
# dialog_wine class
#######################################################################
package Dialog_wine;

use constant FALSE => 0;
use constant TRUE  => 1;

BEGIN {
    use Gnome2;
    use base qw( Generic );
}

sub mt(@) {$AppGwine::lh->maketext(@_)}

sub new {
    my $that = shift;
    my $class = ref($that) || $that;
    my $self = Dialog_wine->SUPER::new();
    bless $self, $class;

    my $title = shift;

    # Construct Dialog
    my ($form);

    $form->{'dialog_wine'} = Gtk2::Dialog->new_with_buttons($title,
							    undef,
							    'modal',
							    'gtk-cancel',
							    'cancel',
							    'gtk-ok',
							    'ok'
							   );
    $form->{'dialog_wine'}->set_border_width(12);
    $form->{'dialog_wine'}->vbox->set_spacing(6);
    $form->{'tips'} = Gtk2::Tooltips->new;
    my $table = $form->{'table1'} = Gtk2::Table->new(7, 5, FALSE);
    $table->set_col_spacings(12);
    $table->set_row_spacings(4);
    $form->{'dialog_wine'}->vbox->pack_start($form->{'table1'}, FALSE, FALSE, 0);
    $form->{'label_domain'} = Gtk2::Label->new_with_mnemonic(mt('_Domain'));
    $form->{'label_domain'}->set_alignment(0, 0.5);
    $form->{'entry_domain'} = Gtk2::Entry->new;
    $form->{'tips'}->set_tip($form->{'entry_domain'}, mt("Wine's name"));
    $form->{'label_domain'}->set_mnemonic_widget($form->{'entry_domain'});
    $table->attach($form->{'label_domain'}, 0, 1, 0, 1,
		  'fill', 'expand', 0, 0);
    $table->attach($form->{'entry_domain'}, 1, 2, 0, 1,
		  'expand', 'expand', 0, 0);
    $form->{'label_vintage'} = Gtk2::Label->new_with_mnemonic(mt('_Vintage'));
    $form->{'label_vintage'}->set_alignment(0, 0.5);
    $form->{'entry_vintage'} = Gtk2::Entry->new;
    $form->{'tips'}->set_tip($form->{'entry_vintage'}, mt('Production year'));
    $form->{'label_vintage'}->set_mnemonic_widget($form->{'entry_vintage'});
    $table->attach($form->{'label_vintage'}, 0, 1, 1, 2,
		  'fill', 'expand', 0, 0);
    $table->attach($form->{'entry_vintage'}, 1, 2, 1, 2,
		  'expand', 'expand', 0, 0);
    $form->{'label_color'} = Gtk2::Label->new_with_mnemonic(mt('_Color'));
    $form->{'label_color'}->set_alignment(0, 0.5);
    $form->{'omenu_color'} = Gtk2::OptionMenu->new;
    $form->{'menu_color'} = Gtk2::Menu->new;
    my @om_uiinfo = map { { type        => 'item',
			    label       => $_,
			  }
		        } @wine_colors;
    $form->{'menu_color'}->append(\@om_uiinfo);
    $form->{'omenu_color'}->set_menu($form->{'menu_color'});
    $form->{'tips'}->set_tip($form->{'omenu_color'}, mt('Wine\'s color'));
    $form->{'label_color'}->set_mnemonic_widget($form->{'omenu_color'});
    $table->attach($form->{'label_color'}, 0, 1, 2, 3,
		  'fill', 'expand', 0, 0);
    $table->attach($form->{'omenu_color'}, 1, 2, 2, 3,
		  'fill', 'expand', 0, 0);
    $form->{'label_quantity'} = Gtk2::Label->new_with_mnemonic(mt('_Quantity'));
    $form->{'label_quantity'}->set_alignment(0, 0.5);
    $form->{'entry_quantity'} = Gtk2::Entry->new;
    $form->{'tips'}->set_tip($form->{'entry_quantity'}, mt('Bottle number'));
    $form->{'label_quantity'}->set_mnemonic_widget($form->{'entry_quantity'});
    $table->attach($form->{'label_quantity'}, 0, 1, 3, 4,
		  'fill', 'expand', 0, 0);
    $table->attach($form->{'entry_quantity'}, 1, 2, 3, 4,
		  'expand', 'expand', 0, 0);
    $form->{'label_size'} = Gtk2::Label->new_with_mnemonic(mt('_Size'));
    $form->{'label_size'}->set_alignment(0, 0.5);
    $form->{'entry_size'} = Gtk2::Entry->new;
    $form->{'tips'}->set_tip($form->{'entry_size'}, mt('Bottle size in liter'));
    $form->{'label_size'}->set_mnemonic_widget($form->{'entry_size'});
    $table->attach($form->{'label_size'}, 0, 1, 4, 5,
		  'fill', 'expand', 0, 0);
    $table->attach($form->{'entry_size'}, 1, 2, 4, 5,
		  'expand', 'expand', 0, 0);
    $form->{'label_note'} = Gtk2::Label->new_with_mnemonic(mt('_Notes'));
    $form->{'label_note'}->set_alignment(0, 0.5);
    $form->{'tview_note'} = Gtk2::TextView->new;
    $form->{'tview_note'}->set('wrap-mode', 'word');
    $form->{'label_note'}->set_mnemonic_widget($form->{'tview_note'});
    $form->{'scrolled_note'} = Gtk2::ScrolledWindow->new;
    $form->{'scrolled_note'}->add($form->{'tview_note'});
    $form->{'scrolled_note'}->set_policy('automatic', 'automatic');
    $table->attach($form->{'label_note'}, 0, 1, 5, 6,
		  'fill', 'expand', 0, 0);
    $table->attach($form->{'scrolled_note'}, 1, 5, 5, 7,
		  'fill', 'fill', 0, 0);
    $form->{'label_bdate'} = Gtk2::Label->new_with_mnemonic(mt('Buying _Date'));
    $form->{'label_bdate'}->set_alignment(0, 0.5);
    $form->{'entry_bdate'} = Gtk2::Entry->new;
    $form->{'tips'}->set_tip($form->{'entry_bdate'}, mt('Buying date (mm-dd-yyyy)'));
    $form->{'label_bdate'}->set_mnemonic_widget($form->{'entry_bdate'});
    $table->attach($form->{'label_bdate'}, 3, 4, 0, 1,
		  'fill', 'expand', 0, 0);
    $table->attach($form->{'entry_bdate'}, 4, 5, 0, 1,
		  'expand', 'expand', 0, 0);
    $form->{'label_cons'} = Gtk2::Label->new_with_mnemonic(mt('C_onservation'));
    $form->{'label_cons'}->set_alignment(0, 0.5);
    $form->{'entry_cons'} = Gtk2::Entry->new;
    $form->{'tips'}->set_tip($form->{'entry_cons'}, mt('Number of year to keep before drinking'));
    $form->{'label_cons'}->set_mnemonic_widget($form->{'entry_cons'});
    $table->attach($form->{'label_cons'}, 3, 4, 1, 2,
		  'fill', 'expand', 0, 0);
    $table->attach($form->{'entry_cons'}, 4, 5, 1, 2,
		  'expand', 'expand', 0, 0);
    $form->{'label_prod'} = Gtk2::Label->new_with_mnemonic(mt('_Productor'));
    $form->{'label_prod'}->set_alignment(0, 0.5);
    $form->{'combo_prod'} = Gtk2::Combo->new;
    $form->{'combo_prod'}->set_value_in_list(TRUE, TRUE);
    $form->{'label_prod'}->set_mnemonic_widget($form->{'combo_prod'});
    $table->attach($form->{'label_prod'}, 3, 4, 2, 3,
		  'fill', 'expand', 0, 0);
    $table->attach($form->{'combo_prod'}, 4, 5, 2, 3,
		  'expand', 'expand', 0, 0);
    $form->{'label_price'} = Gtk2::Label->new_with_mnemonic(mt('Pr_ice'));
    $form->{'label_price'}->set_alignment(0, 0.5);
    $form->{'entry_price'} = Gtk2::Entry->new;
    $form->{'tips'}->set_tip($form->{'entry_price'}, mt('Bottle price'));
    $form->{'label_price'}->set_mnemonic_widget($form->{'entry_price'});
    $table->attach($form->{'label_price'}, 3, 4, 3, 4,
		  'fill', 'expand', 0, 0);
    $table->attach($form->{'entry_price'}, 4, 5, 3, 4,
		  'expand', 'expand', 0, 0);
    

    $self->FORM($form);
    $self->TOPLEVEL($form->{'dialog_wine'});
    return $self;
}


#######################################################################
# dialog_prod class
#######################################################################
package Dialog_prod;

use constant FALSE => 0;
use constant TRUE  => 1;

BEGIN {
    use Gnome2;
    use base qw( Generic );
}

sub mt(@) {$AppGwine::lh->maketext(@_)}

sub new {
    my $that = shift;
    my $class = ref($that) || $that;
    my $self = Dialog_prod->SUPER::new();
    bless $self, $class;

    my $title = shift;

    # Construct Dialog
    my ($form);
    $form->{'dialog_prod'} = Gtk2::Dialog->new_with_buttons($title,
							    undef,
							    'modal',
							    'gtk-cancel',
							    'cancel',
							    'gtk-ok',
							    'ok'
							   );
    $form->{'dialog_prod'}->set_border_width(12);
    $form->{'dialog_prod'}->vbox->set_spacing(6);
    my $table = $form->{'table1'} = Gtk2::Table->new(5, 2, FALSE);
    $table->set_col_spacings(12);
    $table->set_row_spacings(4);
    $form->{'dialog_prod'}->vbox->pack_start($form->{'table1'}, FALSE, FALSE, 0);
    $form->{'label_name'} = Gtk2::Label->new_with_mnemonic(mt('_Name'));
    $form->{'label_name'}->set_alignment(0, 0.5);
    $form->{'entry_name'} = Gtk2::Entry->new;
    $form->{'label_name'}->set_mnemonic_widget($form->{'entry_name'});
    $table->attach($form->{'label_name'}, 0, 1, 0, 1,
		  'fill', 'expand', 0, 0);
    $table->attach($form->{'entry_name'}, 1, 2, 0, 1,
		  'expand', 'expand', 0, 0);
    $form->{'label_addr'} = Gtk2::Label->new_with_mnemonic(mt('_Address'));
    $form->{'label_addr'}->set_alignment(0, 0.5);
    $form->{'entry_addr'} = Gtk2::Entry->new;
    $form->{'label_addr'}->set_mnemonic_widget($form->{'entry_addr'});
    $table->attach($form->{'label_addr'}, 0, 1, 1, 2,
		  'fill', 'expand', 0, 0);
    $table->attach($form->{'entry_addr'}, 1, 2, 1, 2,
		  'expand', 'expand', 0, 0);
    $form->{'label_zip'} = Gtk2::Label->new_with_mnemonic(mt('_Zip Code'));
    $form->{'label_zip'}->set_alignment(0, 0.5);
    $form->{'entry_zip'} = Gtk2::Entry->new;
    $form->{'label_zip'}->set_mnemonic_widget($form->{'entry_zip'});
    $table->attach($form->{'label_zip'}, 0, 1, 2, 3,
		  'fill', 'expand', 0, 0);
    $table->attach($form->{'entry_zip'}, 1, 2, 2, 3,
		  'expand', 'expand', 0, 0);
    $form->{'label_city'} = Gtk2::Label->new_with_mnemonic(mt('_City'));
    $form->{'label_city'}->set_alignment(0, 0.5);
    $form->{'entry_city'} = Gtk2::Entry->new;
    $form->{'label_city'}->set_mnemonic_widget($form->{'entry_city'});
    $table->attach($form->{'label_city'}, 0, 1, 3, 4,
		  'fill', 'expand', 0, 0);
    $table->attach($form->{'entry_city'}, 1, 2, 3, 4,
		  'expand', 'expand', 0, 0);
    $form->{'label_country'} = Gtk2::Label->new_with_mnemonic(mt('C_ountry'));
    $form->{'label_country'}->set_alignment(0, 0.5);
    $form->{'entry_country'} = Gtk2::Entry->new;
    $form->{'label_country'}->set_mnemonic_widget($form->{'entry_country'});
    $table->attach($form->{'label_country'}, 0, 1, 4, 5,
		  'fill', 'expand', 0, 0);
    $table->attach($form->{'entry_country'}, 1, 2, 4, 5,
		  'expand', 'expand', 0, 0);

    $self->FORM($form);
    $self->TOPLEVEL($form->{'dialog_prod'});
    return $self;
}


#######################################################################
# dialog_properties class
#######################################################################
package Dialog_properties;

use constant FALSE => 0;
use constant TRUE  => 1;

BEGIN {
    use Gnome2;
    use base qw( Generic );
}

sub mt(@) {$AppGwine::lh->maketext(@_)}

sub new {
    my $that = shift;
    my $class = ref($that) || $that;
    my $self = Dialog_properties->SUPER::new();
    bless $self, $class;

    my $filename = shift;
    my $date = shift;
    my $wquantity = shift;
    my $bquantity = shift;

    # Construct Dialog
    my ($form);
    $form->{'dialog_prod'} = Gtk2::Dialog->new_with_buttons(mt('Properties of [_1]', $filename),
							    undef, 'modal',
							    'gtk-close',
							    'close');
    $form->{'dialog_prod'}->set_border_width(12);
    $form->{'dialog_prod'}->vbox->set_spacing(6);
    my $table = $form->{'table1'} = Gtk2::Table->new(5, 2, FALSE);
    $table->set_col_spacings(12);
    $table->set_row_spacings(4);
    $form->{'dialog_prod'}->vbox->pack_start($table, FALSE, FALSE, 0);
    $form->{'label_wine'} = Gtk2::Label->new(mt('Wines:'));
    $form->{'label_wine'}->set_alignment(0, 0.5);
    $form->{'label_wquant'} = Gtk2::Label->new($wquantity);
    $table->attach($form->{'label_wine'}, 0, 1, 0, 1,
		  'fill', 'expand', 0, 0);
    $table->attach($form->{'label_wquant'}, 1, 2, 0, 1,
		  'expand', 'expand', 0, 0);
    $form->{'label_bottle'} = Gtk2::Label->new(mt('Bottles:'));
    $form->{'label_bottle'}->set_alignment(0, 0.5);
    $form->{'label_bquant'} = Gtk2::Label->new($bquantity);
    $table->attach($form->{'label_bottle'}, 0, 1, 1, 2,
		  'fill', 'expand', 0, 0);
    $table->attach($form->{'label_bquant'}, 1, 2, 1, 2,
		  'expand', 'expand', 0, 0);
#    $form->{'separator'} = Gtk2::HSeparator->new;
#    $table->attach($form->{'separator'}, 0, 1, 2, 3,
#		  'fill', 'expand', 0, 0);
    $form->{'label_date'} = Gtk2::Label->new(mt('Last modification date:'));
    $form->{'label_date'}->set_alignment(0, 0.5);
    $form->{'label_date2'} = Gtk2::Label->new($date);
    $table->attach($form->{'label_date'}, 0, 2, 3, 4,
		  'fill', 'expand', 0, 0);
    $table->attach($form->{'label_date2'}, 1, 2, 4, 5,
		  'expand', 'expand', 0, 0);

    $self->FORM($form);
    $self->TOPLEVEL($form->{'dialog_prod'});
    return $self;
}



#######################################################################
# dialog_preferences class
#######################################################################
package Dialog_preferences;

use constant FALSE => 0;
use constant TRUE  => 1;

BEGIN {
    use Gnome2;
    use base qw( Generic );
}

sub mt(@) {$AppGwine::lh->maketext(@_)}

sub new {
    my $that = shift;
    my $class = ref($that) || $that;
    my $self = Dialog_preferences->SUPER::new();
    bless $self, $class;

    # Construct Dialog
    my ($form);
    $form->{'dialog_pref'} = Gtk2::Dialog->new_with_buttons(mt('Preferences'),
							    undef, 'modal',
							    'gtk-close',
							    'close');
    $form->{'dialog_pref'}->set_border_width(12);
    $form->{'dialog_pref'}->vbox->set_spacing(6);
    $form->{'notebook'} = Gtk2::Notebook->new;
    $form->{'dialog_pref'}->vbox->pack_start($form->{'notebook'},
					     FALSE, FALSE, 0);

    # Set General page
    my $vbox = $form->{'vbox1'} = Gtk2::VBox->new(FALSE, 4);
    $form->{button_lfile} = Gtk2::CheckButton->new(mt('Open last opened _file at startup'));
    $vbox->pack_start($form->{'button_lfile'}, FALSE, FALSE, 0);
    $form->{button_size} = Gtk2::CheckButton->new(mt('Save application _size between sessions'));
    $vbox->pack_start($form->{'button_size'}, FALSE, FALSE, 0);
    $form->{button_popup} = Gtk2::CheckButton->new(mt('Use confirmation _popups'));
    $vbox->pack_start($form->{'button_popup'}, FALSE, FALSE, 0);
    my $lg = Gtk2::Label->new(mt('General'));
    $form->{'notebook'}->append_page($vbox, $lg);

    # Set Wines page
    $form->{'vbox2'} = Gtk2::VBox->new(FALSE, 4);
    $form->{'lstore1'} = Gtk2::ListStore->new('Glib::String', 'Glib::Boolean');
    $form->{'tview1'} = Gtk2::TreeView->new_with_model($form->{'lstore1'});
    my $renderer = Gtk2::CellRendererText->new;
    my $col = Gtk2::TreeViewColumn->new_with_attributes(mt('Columns'), 
							$renderer,
							'text' => 0);
    $form->{'tview1'}->append_column($col);
    $renderer = Gtk2::CellRendererToggle->new;
    $renderer->signal_connect (toggled => "$class\::on_toggled_event", 
			       $form->{'lstore1'});
    $col = Gtk2::TreeViewColumn->new_with_attributes(mt('Visible'), 
							$renderer,
							'active' => 1);
    $form->{'tview1'}->append_column($col);
    $form->{'tview1'}->get_selection->set_mode('single');

    $form->{'label1'} = Gtk2::Label->new(mt('_Show wine list columns'));
    $form->{'label1'}->set_mnemonic_widget($form->{'tview1'});
    $form->{'vbox2'}->pack_start($form->{'label1'}, FALSE, FALSE, 4);
    $form->{'vbox2'}->pack_start($form->{'tview1'}, FALSE, FALSE, 4);
    $form->{'notebook'}->append_page($form->{'vbox2'}, mt('Wines'));
    

    # Set Productors page
    $form->{'vbox3'} = Gtk2::VBox->new(FALSE, 4);
    $form->{'lstore2'} = Gtk2::ListStore->new('Glib::String', 'Glib::Boolean');
    $form->{'tview2'} = Gtk2::TreeView->new_with_model($form->{'lstore2'});
    $renderer = Gtk2::CellRendererText->new;
    $col = Gtk2::TreeViewColumn->new_with_attributes(mt('Columns'), 
							$renderer,
							'text' => 0);
    $form->{'tview2'}->append_column($col);
    $renderer = Gtk2::CellRendererToggle->new;
    $renderer->signal_connect (toggled => "$class\::on_toggled_event", 
			       $form->{'lstore2'});
    $col = Gtk2::TreeViewColumn->new_with_attributes(mt('Visible'), 
							$renderer,
							'active' => 1);
    $form->{'tview2'}->append_column($col);
    $form->{'tview2'}->get_selection->set_mode('single');

    $form->{'label2'} = Gtk2::Label->new(mt('_Show productor list columns'));
    $form->{'label2'}->set_mnemonic_widget($form->{'tview2'});
    $form->{'vbox3'}->pack_start($form->{'label2'}, FALSE, FALSE, 4);
    $form->{'vbox3'}->pack_start($form->{'tview2'}, FALSE, FALSE, 4);
    $form->{'notebook'}->append_page($form->{'vbox3'}, mt('Productors'));


    $self->FORM($form);
    $self->TOPLEVEL($form->{'dialog_pref'});
    return $self;
}


1;

__END__
