#
#   Makefile.PL for Gwine2
#    - Primary version make by Glade-Perl
#    - Modify by S. Daubert

eval { require 5.008 } or die <<EOD;
######################################################
## Gwine uses version 5.8.0 of Perl.                ##
## Please upgrade your Perl before installing Gwine ##
######################################################
EOD

use ExtUtils::MakeMaker;
use strict;
use warnings;

#--- Configuration section ---

my @programs_to_install = qw( gwine );

my @need_perl_modules = (
    # Check for Glib
    {'name'     => 'Glib',
    'test'      => 'Glib',
    'version'   => '1.00',
    'reason'    => "implements the perl bindings to Glib 2.x.\n".
                    "The module is called Glib-Perl on CPAN"},

    # Check for Gtk2
    {'name'     => 'Gtk2',
    'test'      => 'Gtk2',
    'version'   => '1.00',
    'reason'    => "implements the perl bindings to Gtk+ 2.x.\n".
                    "The module is called Gtk2-Perl on CPAN"},

    # Check for Gnome2
    {'name'     => 'Gnome2',
    'test'      => 'Gnome2',
    'version'   => '0.38',
    'reason'    => "implements the perl bindings to Gnome 2.x.\n".
                   "It is a submodule of the Gtk2-Perl package and needs to be built separately.\n".
                   "Read the Gtk2-Perl INSTALL file for details of how to do this."},
#    # Check for Number::Format
#    {'name'     => 'Number::Format',
#    'test'      => 'Number::Format',
#    'version'   => '1.45',
#    'reason'    => "formats number according to locale"},
    # Check for DateTime
    {'name'     => 'DateTime',
    'test'      => 'DateTime',
    'version'   => '0.1601',
    'reason'    => "manipulate date and time according to locale"},
    );

#--- End Configuration - You should not have to change anything below this line

# Allow us to suppress all program installation with the -n (library only)
# option.  This is for those that don't want to mess with the configuration
# section of this file.
use Getopt::Std;
use vars qw($opt_n);
unless (getopts("n")) {
    die "Usage: $0 [-n]\n";
}
@programs_to_install = () if $opt_n;

# Check for non-standard modules that are used by this library.
$| = 1; # autoflush on
my $missing_modules = 0;

foreach my $mod (@need_perl_modules) {
    print "Checking for $mod->{'name'}..";
    eval "use $mod->{'test'} $mod->{'version'}";
    if ($@) {
        $missing_modules++;
		print " failed\n";
        print   "-------------------------------------------------------".
                "\n$@\n",
                "$mod->{'name'} is needed, it $mod->{'reason'}\n",
                "We need at least version $mod->{'version'}\n".
                "-------------------------------------------------------\n";
        sleep(2);  # Don't hurry too much
    } else {
		print " ok\n";
    }
}

#--------------------------------------
print "-------------------------------------------------------
The missing modules can be obtained from CPAN. Visit
<URL:http://www.perl.com/CPAN/> to find a CPAN site near you.
-------------------------------------------------------\n\n"
     if $missing_modules;

#--------------------------------------
# Last of all generate the Makefile
WriteMakefile(
    'DISTNAME'     => 'gwine',
    'NAME'         => 'gwine',
    'VERSION_FROM' => 'Gwine/gwineUI.pm',
    'EXE_FILES'    => [ @programs_to_install ],
    'PREFIX'       => '/usr/local',
    'INSTALLDIRS'  => 'perl',
    'clean'        => { FILES => '' },
    'dist'         => { COMPRESS => 'gzip', SUFFIX => 'gz' },
    'PM'           => {
	'Gwine/Config.pm' => '$(INST_LIBDIR)/Gwine/Config.pm',
	'Gwine/GAppel.pm' => '$(INST_LIBDIR)/Gwine/GAppel.pm',
	'Gwine/GWine.pm' => '$(INST_LIBDIR)/Gwine/GWine.pm',
	'Gwine/GProd.pm' => '$(INST_LIBDIR)/Gwine/GProd.pm',
	'Gwine/GList.pm' => '$(INST_LIBDIR)/Gwine/GList.pm',
	'Gwine/GLAppel.pm' => '$(INST_LIBDIR)/Gwine/GLAppel.pm',
	'Gwine/GLWine.pm' => '$(INST_LIBDIR)/Gwine/GLWine.pm',
	'Gwine/GLProd.pm' => '$(INST_LIBDIR)/Gwine/GLProd.pm',
	'Gwine/GZone.pm' => '$(INST_LIBDIR)/Gwine/GZone.pm',
	'Gwine/GLZone.pm' => '$(INST_LIBDIR)/Gwine/GLZone.pm',
	'Gwine/Gdb.pm' => '$(INST_LIBDIR)/Gwine/Gdb.pm',
	'Gwine/gwine.pm' => '$(INST_LIBDIR)/Gwine/gwine.pm',
	'Gwine/gwineUI.pm' => '$(INST_LIBDIR)/Gwine/gwineUI.pm',
	'Gwine/L10N.pm' => '$(INST_LIBDIR)/Gwine/L10N.pm',
	'Gwine/L10N/en.pm' => '$(INST_LIBDIR)/Gwine/L10N/en.pm',
	'Gwine/L10N/fr.pm' => '$(INST_LIBDIR)/Gwine/L10N/fr.pm',
	'Gwine/L10N/it.pm' => '$(INST_LIBDIR)/Gwine/L10N/it.pm',
	'Changelog' => '$(INST_LIBDIR)/Gwine/Changelog',
	'Changelog.old' => '$(INST_LIBDIR)/Gwine/Changelog.old',
	'COPYING' => '$(INST_LIBDIR)/Gwine/COPYING',
	'INSTALL' => '$(INST_LIBDIR)/Gwine/INSTALL',
	'README'  => '$(INST_LIBDIR)/Gwine/README',
	'TODO'    => '$(INST_LIBDIR)/Gwine/TODO'
	}
);

package MY;

# Pass version number to pod2man
sub manifypods
{
    my $self = shift;
    my $ver = $self->{'VERSION'} || "";
    local($_) = $self->SUPER::manifypods(@_);
    s/pod2man\s*$/pod2man --release gwine-$ver/m;
    $_;
}

sub install {
    my $self = shift;
    local($_) = $self->SUPER::install(@_);
    s/install ::/install:: post_install/m;
    s/uninstall ::/uninstall:: post_uninstall/m;
    $_;
}

sub postamble {
    my $self = shift;
    my $prefix = $self->{PREFIX};

    print "Setting pixmaps directory in gwine...";
    my $path = "$prefix/share/pixmaps";
    $path =~ s{//}{/};
    my $subs = 's{\$config->icon_path\(.*\)}{\$config->icon_path\("' . "$path" . '"\)}';
    system("perl -pi -e '$subs' gwine");
    print "$path\n";
    my $pixmaps_install = "\tmkdir -p $path\n";
    my $pixmaps_uninstall = "";
    foreach my $file (<pixmaps/*.png>) {
	$pixmaps_install .= "\tcp $file $path\n";
	$pixmaps_uninstall .= "\trm -f $path/$file\n";	
    }

#    my %locales;
#    my @locales = `locale`;
#    foreach (@locales) {
#	$locales{$1}++ if /=\"(.+)\"/;
#    }
#    print "locales.......";
#    foreach (keys %locales) { print "$_ " if $locales{$_} > 0}
#    print "\n";

#    my ($locales_install, $locales_uninstall) = ("", "");
#    foreach my $locale (keys %locales) {
#	if ( index($locale, 'fr') != -1 ) {
#	    $locales_install .= "\tmkdir -p $prefix/share/locale/$locale/LC_MESSAGES\n";
#	    $locales_install .= "\tcp po/fr.mo $prefix/share/locale/$locale/LC_MESSAGES/gwine.mo\n";
#	    $locales_uninstall .= "\trm -f $prefix/share/locale/$locale/LC_MESSAGES/gwine.mo\n"
#	    }
#    }

#<<EOT;
#post_install:
#$pixmaps_install
#$locales_install
#
#post_uninstall:
#$pixmaps_uninstall
#$locales_uninstall
#EOT
<<EOT;
post_install:
$pixmaps_install

post_uninstall:
$pixmaps_uninstall
EOT
}


exit(0);

# End of Makefile.PL
